@@ -1,3 +1,4 @@
+use super::supertrait_gen;
 use super::util::*;
 use crate::model::{ContractTrait, Method, PublicRole};
 
@@ -14,10 +15,7 @@ fn function_selector_match_arm(m: &Method, endpoint_name: &str) -> proc_macro2::
 	}
 }
 
-pub fn generate_function_selector_body(
-	contract: &ContractTrait,
-	include_submodules: bool,
-) -> proc_macro2::TokenStream {
+pub fn generate_function_selector_body(contract: &ContractTrait) -> proc_macro2::TokenStream {
 	let match_arms: Vec<proc_macro2::TokenStream> = contract
 		.methods
 		.iter()
@@ -30,36 +28,20 @@ pub fn generate_function_selector_body(
 			_ => None,
 		})
 		.collect();
-
-	let module_arms: Vec<proc_macro2::TokenStream> = if include_submodules {
-		contract
-			.methods
-			.iter()
-			.filter_map(|m| {
-				if m.is_module() {
-					let method_name = &m.name;
-					Some(quote! {
-						if self.#method_name().call(fn_name) {
-							return true;
-						}
-					})
-				} else {
-					None
-				}
-			})
-			.collect()
-	} else {
-		Vec::new()
-	};
+	let module_calls =
+		supertrait_gen::function_selector_module_calls(contract.supertraits.as_slice());
 	quote! {
 		if match fn_name {
-			b"callBack" => { self.callback(); return true; }
+			b"callBack" => {
+				self::EndpointWrappers::callback(self);
+				return true;
+			}
 			#(#match_arms)*
 			other => false
 		} {
 			return true;
 		}
-		#(#module_arms)*
+		#(#module_calls)*
 		false
 	}
 }
